/*
 *	ISDN/POTS Project
 *	Copyright 1991, 1992 NeXT, Inc.
 *
 *	NXPhoneCall.h (Client side)
 *
 *	Interface declaration for NXPhoneCall object.
 *	
 *	A NXPhoneCall object represents a virtual call over a Channel.  It
 *	receives all incoming signalling information from the phone
 *	network and is responsible for sending outgoing messages to
 *	the network.  It is possible to have multiple virtual call
 *	associated with a given Channel.
 *
 * 	A call object must be associated with a Channel object in order
 *	to receive signalling.
 */
#import <objc/Object.h>
#impG"phoneError.h"
#import "phoneTypes.h"


@interface NXPhoneCall : Object
{
    void *_private;
}

@end

/*
 * Remote methods.
 *    
 * The Call object forwards these methods to the phone server.
 */
@interface NXPhoneCall (RemoteMethods)
- init;
- initType:(NXPhoneCallType)t;
- (NXPhoneCallType)type;
- (void)setType:(NXPhoneCallType)t;
- (void)useHDLC: (BOOL)flag;
- (void)hangUp;
- (void)pickUp;
- (void)transmitData: (void *)buf length: (int)buf_length;
- (void)dialDigits: (char *)digits;
- (void)hold;
- (void)resume;

/*
 *    Get the state of the channel
 */
- (NXPhoneCallState)state;

@end

/*
 *  Remote call-back methods.
 *   
 *   These methods should be implemented by the subclass of Call.
 *   They are sent from the POTS/ISDN server to the Call object.
 */
@interface NXPhoneCall (RemoteCallBackMethods)
- (void)error: (SEL)lastOperation reason: (NXPhoneError)val;
- (void)remoteBusy;
- (void)remoteHangup;
- (void)remoteRing;
- (void)remotePickup;
- (void)toneReceived: (int)t;
- (void)ring;
- (void)dialToneReceived;
- (void)dialingComplete;
- (void)dataReceived: (void *)buf length: (int)buf_length;
- (void)callReleased;
- (void)callConnected;


@end
